{% if not resource %}
<a name="{{ app|slugify }}-{{ version|slugify }}-{{ descriptor.service_name|slugify }}"><h3>{{ descriptor.service_name }}</h3></a><span class="tiny"><a href="#">Back to Top</a></span>
<h4>{{ api_root }}/{{ app }}/{{ version }}/{{ descriptor.slug }}/</h4>
{% if descriptor.is_deprecated %}
<h4>DEPRECATED: {{ descriptor.is_deprecated }}</h4>
{% endif %}
{% endif %}
{% if descriptor.data_parsing %}
<p><em>This is a data parsing function.  Params must be posted as serialized data.</em></p>
{% endif %}
{% if descriptor.security.permissions %}
<p><em>Permissions: {{ descriptor.security.permissions|join:" " }}</em></p>
{% endif %}
<div class="docs">
	{{ descriptor.docs|safe }}
</div>
{% if descriptor.params %}
<table class="params">
	<tr>
		<th>Parameter</th>
		<th>Type</th>
		<th>Required</th>
		<th>Default</th>
		<th>Description</th>
	</tr>
	{% for param in descriptor.params %}
	<tr>
		<td>{{ param.name }}</td>
		<td>{{ param.type }}</td>
		<td>{{ param.required|yesno:"Required,Not Required"}}</td>
		<td>{{ param.default }}</td>
		<td>{{ param.description }}</td>
	</tr>
	{% endfor %}
</table>
{% endif %}